==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/server/saml/SAMLNameIdMappingImpl.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.saml;

import java.util.List;

import javax.jws.WebService;

import org.apache.log4j.Logger;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.samly2.exceptions.SAMLProtocolException;
import eu.unicore.samly2.exceptions.SAMLRequestException;
import eu.unicore.samly2.proto.NameIDMappingResponse;
import eu.unicore.security.dsig.DSigException;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.AuthenticationException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.engine.InternalAuthorization;
import pl.edu.icm.unicore.uvos.engine.api.UVOSQueryInterface;
import pl.edu.icm.unicore.uvos.saml.SAMLConfiguration;
import pl.edu.icm.unicore.uvos.saml.SAMLNameMappingUtil;
import pl.edu.icm.unicore.uvos.server.CryptoDataProvider;
import pl.edu.icm.unicore.uvos.server.idresolver.IdResolveException;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLNameIdMappingInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.org.oasis.saml2.assertion.NameIDType;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingResponseDocument;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDEntryType;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDsDocument;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDsType;

/**
 * @author K. Benedyczak
 */
@WebService(endpointInterface="pl.edu.icm.unicore.uvos.wsapi.SAMLNameIdMappingInterface")
public class SAMLNameIdMappingImpl implements SAMLNameIdMappingInterface
{
	private static final Logger log = Logger.getLogger(SAMLNameIdMappingImpl.class);
	private UVOSQueryInterface voQuery;
	private int reqValidity;
	private Configuration conf;
	private CryptoDataProvider cryptoProvider;
	private Authenticator authenticator;
	private InternalAuthorization authz;
	private SAMLConfiguration samlConfiguration;

	public SAMLNameIdMappingImpl(UVOSQueryInterface voQuery, Configuration conf,
			CryptoDataProvider cryptoProvider, Authenticator authenticator, InternalAuthorization authz,
			SAMLConfiguration samlConfig)
	{
		this.samlConfiguration=samlConfig;
		this.voQuery = voQuery;
		this.conf = conf;
		this.cryptoProvider = cryptoProvider;
		this.authenticator = authenticator;
		this.authz = authz;
		reqValidity = conf.getAsInt(
				ConfDefaults.SAML_REQUEST_VALIDITY, 
				ConfDefaults.samlRequestValidity, log);
	}
	
	/**
	 * Implementation of SAML nameId mapping protocol. It is used to convert format of
	 * possessed nameId into another, or from the UVOS point of view to get information
	 * about equivalent identities.
	 * <p>
	 * At first the format in question is evaluated. If it is "unspecified" then this
	 * SAML request is translated into getAllEquivalents. If format is concrete then
	 * only the first one of found equivalents (if any) is returned, so it is a standard 
	 * SAML behavior.
	 * <p>
	 * In case when more than one result is returned, all results are put into extensions
	 * element of response. 
	 * @throws IdResolveException 
	 * @throws InternalException 
	 *   
	 */
	public NameIDMappingResponseDocument mapNameId(NameIDMappingRequestDocument reqDoc) 
		throws UVOSFault
	{
		//CryptoDataProvider cryptoProvider = VOServer.getCryptoProvider();
		SAMLNameMappingUtil samlUtil = new SAMLNameMappingUtil(reqDoc, cryptoProvider, conf, samlConfiguration);
		try
		{
			authenticator.resolve();
			samlUtil.parse();
			samlUtil.checkRequestValidity(reqValidity);
		} catch (IdResolveException e)
		{
			log.debug("UVOS engine problem while processing " +
					"SAML request (authN stage )", e);
			throw new UVOSFault(e);
		} catch (UVOSException e)
		{
			log.debug("Authentication FAILED: " + e);
			String info = "Authentication failed: " + ((e.getMessage() == null) ? 
					e.getClass().getSimpleName() : e.getMessage());
			return samlUtil.getErrorResponse(new AuthenticationException(), 
					info).getDoc();
		} catch (SAMLProtocolException e1)
		{
			log.debug("Parsing the SAML request FAILED: " + e1);
			return samlUtil.getErrorResponse(e1).getDoc();
		}
		
		String format = samlUtil.getRequestedFormat();
		Identity originalId = samlUtil.getOriginalIdentity();
		
		//InternalAuthorization authz = Manager.getInstance().getInternalAuthz();
		try
		{
			if (!authz.isActive(originalId))
			{
				SAMLRequestException e = new SAMLRequestException(
						SAMLConstants.STATUS2_UNKNOWN_PRINCIPIAL,
						"Query subject is unknown");
				return samlUtil.getErrorResponse(e).getDoc();
			}
		} catch (UVOSException e1)
		{
			log.debug("Query subject is unknown: " + e1);
			return samlUtil.getErrorResponse(e1).getDoc();
		}

		
		List<Identity> equivalents;
		try
		{
			equivalents = voQuery.getAllEquivalents(originalId);
		} catch (InternalException e)
		{
			log.debug("SAML request processing failed (in engine code) : " +
					e);
			throw new UVOSFault(e);
		} catch (UVOSException e)
		{
			log.debug("SAML request processing failed (in engine code) : " +
					e);
			return samlUtil.getErrorResponse(e).getDoc();
		}

		removeSelf(equivalents, originalId);
		
		if (equivalents.size() == 0)
			return samlUtil.getErrorResponse(new SAMLRequestException(
				SAMLConstants.STATUS2_INVALID_NAMEID_POLICY, 
				"No matching alternative names available.")).getDoc();
		
		NameIDMappingResponse resp;
		
		if (format.equals(SAMLConstants.NFORMAT_UNSPEC))
		{
			resp = samlUtil.getOKResponse(
					identity2SAML(equivalents.get(0)));
			addAllToExt(equivalents, resp);
		} else
		{
			filterMatching(equivalents, format);
			if (equivalents.size() == 0)
				return samlUtil.getErrorResponse(
					new SAMLRequestException(
					SAMLConstants.STATUS2_INVALID_NAMEID_POLICY, 
					"No matching alternative names available.")).getDoc();
			resp = samlUtil.getOKResponse(
					identity2SAML(equivalents.get(0)));
		}

		if (samlUtil.doSignResponse())
			try
			{
				resp.sign(cryptoProvider.getPrivateKey(), 
						cryptoProvider.getServerCertificateC());
			} catch (DSigException e)
			{
				log.error("Signing SAML assertion error", e);
				throw new UVOSFault(new InternalException(
					"Signing response assertion problem", e));
			}

		return resp.getDoc();
	}
	
	
	private NameIDType identity2SAML(Identity id, NameIDType samlName)
	{
		samlName.setFormat(id.getType().toString());
		samlName.setStringValue(id.getValue());
		return samlName;
	}

	private NameID identity2SAML(Identity id)
	{
		return new NameID(id.getValue(), id.getType().toString());
	}

	private void removeSelf(List<Identity> list, Identity orig)
	{
		String oF = orig.getType().toString();
		String oV = orig.getValue();
		for (int i=0; i<list.size(); i++)
		{
			Identity id = list.get(i);
			if (id.getType().toString().equals(oF) && 
					id.getValue().equals(oV))
			{
				list.remove(i);
				return;
			}
		}
	}
	
	private void filterMatching(List<Identity> list, String format)
	{
		for (int i=0; i<list.size(); i++)
		{
			Identity id = list.get(i);
			if (!id.getType().toString().equals(format))
			{
				list.remove(i);
				i--;
			}
		}
	}
	
	private void addAllToExt(List<Identity> list, NameIDMappingResponse resp)
	{
		AlternativeNameIDsDocument extDoc = AlternativeNameIDsDocument.
			Factory.newInstance();
		AlternativeNameIDsType altName = extDoc.addNewAlternativeNameIDs();
		for (Identity id: list)
		{
			AlternativeNameIDEntryType entry = altName.addNewMappedNameID();
			NameIDType samlName = entry.addNewNameID(); 
			identity2SAML(id, samlName);
		}
		resp.setExtensions(extDoc);
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/server/saml/SAMLNameIdMappingImpl.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server.saml;

import java.util.List;

import javax.jws.WebService;

import org.apache.log4j.Logger;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.samly2.exceptions.SAMLProtocolException;
import eu.unicore.samly2.exceptions.SAMLRequestException;
import eu.unicore.samly2.proto.NameIDMappingResponse;
import eu.unicore.security.dsig.DSigException;
import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.AuthenticationException;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.engine.InternalAuthorization;
import pl.edu.icm.unicore.uvos.engine.api.UVOSQueryInterface;
import pl.edu.icm.unicore.uvos.saml.SAMLConfiguration;
import pl.edu.icm.unicore.uvos.saml.SAMLNameMappingUtil;
import pl.edu.icm.unicore.uvos.server.IssuerIdentityProvider;
import pl.edu.icm.unicore.uvos.server.idresolver.IdResolveException;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLNameIdMappingInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.org.oasis.saml2.assertion.NameIDType;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.NameIDMappingResponseDocument;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDEntryType;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDsDocument;
import xmlbeans.pl.edu.icm.samlvo.namemapext.AlternativeNameIDsType;

/**
 * @author K. Benedyczak
 */
@WebService(endpointInterface="pl.edu.icm.unicore.uvos.wsapi.SAMLNameIdMappingInterface")
public class SAMLNameIdMappingImpl implements SAMLNameIdMappingInterface
{
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, SAMLNameIdMappingImpl.class);
	private UVOSQueryInterface voQuery;
	private int reqValidity;
	private UVOSServerProperties conf;
	private IssuerIdentityProvider cryptoProvider;
	private Authenticator authenticator;
	private InternalAuthorization authz;
	private SAMLConfiguration samlConfiguration;

	public SAMLNameIdMappingImpl(UVOSQueryInterface voQuery, UVOSServerProperties conf,
			IssuerIdentityProvider cryptoProvider, Authenticator authenticator, InternalAuthorization authz,
			SAMLConfiguration samlConfig)
	{
		this.samlConfiguration=samlConfig;
		this.voQuery = voQuery;
		this.conf = conf;
		this.cryptoProvider = cryptoProvider;
		this.authenticator = authenticator;
		this.authz = authz;
		reqValidity = conf.getIntValue(UVOSServerProperties.SAML_REQUEST_VALIDITY);
	}
	
	/**
	 * Implementation of SAML nameId mapping protocol. It is used to convert format of
	 * possessed nameId into another, or from the UVOS point of view to get information
	 * about equivalent identities.
	 * <p>
	 * At first the format in question is evaluated. If it is "unspecified" then this
	 * SAML request is translated into getAllEquivalents. If format is concrete then
	 * only the first one of found equivalents (if any) is returned, so it is a standard 
	 * SAML behavior.
	 * <p>
	 * In case when more than one result is returned, all results are put into extensions
	 * element of response. 
	 * @throws IdResolveException 
	 * @throws InternalException 
	 *   
	 */
	public NameIDMappingResponseDocument mapNameId(NameIDMappingRequestDocument reqDoc) 
		throws UVOSFault
	{
		//CryptoDataProvider cryptoProvider = VOServer.getCryptoProvider();
		SAMLNameMappingUtil samlUtil = new SAMLNameMappingUtil(reqDoc, cryptoProvider, conf, samlConfiguration);
		try
		{
			authenticator.resolve();
			samlUtil.parse();
			samlUtil.checkRequestValidity(reqValidity);
		} catch (IdResolveException e)
		{
			log.debug("UVOS engine problem while processing " +
					"SAML request (authN stage )", e);
			throw new UVOSFault(e);
		} catch (UVOSException e)
		{
			log.debug("Authentication FAILED: " + e);
			String info = "Authentication failed: " + ((e.getMessage() == null) ? 
					e.getClass().getSimpleName() : e.getMessage());
			return samlUtil.getErrorResponse(new AuthenticationException(), 
					info).getDoc();
		} catch (SAMLProtocolException e1)
		{
			log.debug("Parsing the SAML request FAILED: " + e1);
			return samlUtil.getErrorResponse(e1).getDoc();
		}
		
		String format = samlUtil.getRequestedFormat();
		Identity originalId = samlUtil.getOriginalIdentity();
		
		//InternalAuthorization authz = Manager.getInstance().getInternalAuthz();
		try
		{
			if (!authz.isActive(originalId))
			{
				SAMLRequestException e = new SAMLRequestException(
						SAMLConstants.STATUS2_UNKNOWN_PRINCIPIAL,
						"Query subject is unknown");
				return samlUtil.getErrorResponse(e).getDoc();
			}
		} catch (UVOSException e1)
		{
			log.debug("Query subject is unknown: " + e1);
			return samlUtil.getErrorResponse(e1).getDoc();
		}

		
		List<Identity> equivalents;
		try
		{
			equivalents = voQuery.getAllEquivalents(originalId);
		} catch (InternalException e)
		{
			log.debug("SAML request processing failed (in engine code) : " +
					e);
			throw new UVOSFault(e);
		} catch (UVOSException e)
		{
			log.debug("SAML request processing failed (in engine code) : " +
					e);
			return samlUtil.getErrorResponse(e).getDoc();
		}

		removeSelf(equivalents, originalId);
		
		if (equivalents.size() == 0)
			return samlUtil.getErrorResponse(new SAMLRequestException(
				SAMLConstants.STATUS2_INVALID_NAMEID_POLICY, 
				"No matching alternative names available.")).getDoc();
		
		NameIDMappingResponse resp;
		
		if (format.equals(SAMLConstants.NFORMAT_UNSPEC))
		{
			resp = samlUtil.getOKResponse(
					identity2SAML(equivalents.get(0)));
			addAllToExt(equivalents, resp);
		} else
		{
			filterMatching(equivalents, format);
			if (equivalents.size() == 0)
				return samlUtil.getErrorResponse(
					new SAMLRequestException(
					SAMLConstants.STATUS2_INVALID_NAMEID_POLICY, 
					"No matching alternative names available.")).getDoc();
			resp = samlUtil.getOKResponse(
					identity2SAML(equivalents.get(0)));
		}

		if (samlUtil.doSignResponse())
			try
			{
				resp.sign(conf.getAuthAndTrust().getCredential().getKey(), 
						conf.getAuthAndTrust().getCredential().getCertificateChain());
			} catch (DSigException e)
			{
				log.error("Signing SAML assertion error", e);
				throw new UVOSFault(new InternalException(
					"Signing response assertion problem", e));
			}

		return resp.getDoc();
	}
	
	
	private NameIDType identity2SAML(Identity id, NameIDType samlName)
	{
		samlName.setFormat(id.getType().toString());
		samlName.setStringValue(id.getValue());
		return samlName;
	}

	private NameID identity2SAML(Identity id)
	{
		return new NameID(id.getValue(), id.getType().toString());
	}

	private void removeSelf(List<Identity> list, Identity orig)
	{
		String oF = orig.getType().toString();
		String oV = orig.getValue();
		for (int i=0; i<list.size(); i++)
		{
			Identity id = list.get(i);
			if (id.getType().toString().equals(oF) && 
					id.getValue().equals(oV))
			{
				list.remove(i);
				return;
			}
		}
	}
	
	private void filterMatching(List<Identity> list, String format)
	{
		for (int i=0; i<list.size(); i++)
		{
			Identity id = list.get(i);
			if (!id.getType().toString().equals(format))
			{
				list.remove(i);
				i--;
			}
		}
	}
	
	private void addAllToExt(List<Identity> list, NameIDMappingResponse resp)
	{
		AlternativeNameIDsDocument extDoc = AlternativeNameIDsDocument.
			Factory.newInstance();
		AlternativeNameIDsType altName = extDoc.addNewAlternativeNameIDs();
		for (Identity id: list)
		{
			AlternativeNameIDEntryType entry = altName.addNewMappedNameID();
			NameIDType samlName = entry.addNewNameID(); 
			identity2SAML(id, samlName);
		}
		resp.setExtensions(extDoc);
	}
}
